package pps.core.base.utils;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.enums.SqlMethod;
import com.baomidou.mybatisplus.extension.toolkit.SqlHelper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.QuerySysDictionaryViewInput;
import pps.cloud.system.service.data.QuerySysDictionaryViewOutput;
import pps.core.base.entity.BaseEnt;
import pps.core.base.entity.BasePowerLineEnt;
import pps.core.base.mapper.BasePowerLineMapper;
import pps.core.common.session.PpsUserSession;
import xstartup.base.XContext;
import xstartup.data.XListResult;

import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * 封装工具类
 *
 * @author ZWT
 * @date 2023/08/25 16:08
 */
public class BaseEntUtils {

    /**
     * 设置市电峰谷策略默认值
     *
     * @param entity  实体
     * @param session 会话
     */
    public static void setBaseEntDefault(BaseEnt entity, PpsUserSession session) {
        String id = entity.getId();
        Date now = new Date();
        String userId = session.getId();
        String userName = session.getUserName();
        if (StringUtils.isEmpty(id)) {
            entity.setCreateById(userId);
            entity.setCreateByName(userName);
            entity.setCreateTime(now);
        }
        entity.setModifyById(userId);
        entity.setModifyByName(userName);
        entity.setModifyTime(now);
    }

    /**
     * 批量插入
     *
     * @param mapper      映射器
     * @param entityClass 实体类
     * @param list        列表
     */
    public static <E> void batchInsert(Class<?> mapper, Class<?> entityClass, Collection<E> list) {
        String sqlStatement = SqlHelper.getSqlStatement(mapper, SqlMethod.INSERT_ONE);
        SqlHelper.executeBatch(entityClass, null, list, 1000, (sqlSession, ent) -> {
            sqlSession.insert(sqlStatement, ent);
        });
    }

    /**
     * 通过字典类型查询字典
     *
     * @param context 上下文
     * @param alias   别名
     * @return {@link List}<{@link QuerySysDictionaryViewOutput}>
     */
    public static List<QuerySysDictionaryViewOutput> getDictList(XContext context, String alias) {
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult();
    }

    /**
     * 检查市电峰谷策略是否被引用
     *
     * @param context    上下文
     * @param strategyId 战略id
     * @return boolean
     */
    public static boolean checkStrategyIsReference(XContext context, String strategyId) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLineEnt>()
                .eq(BasePowerLineEnt::getStrategyId, strategyId)
        );
        return count > 0;
    }
}